@@ -1,5 +1,5 @@
 -- DAV.hs: WebDAV client library
--- Copyright © 2012-2014  Clint Adams
+-- Copyright © 2012-2015  Clint Adams
 --
 -- vim: softtabstop=4:shiftwidth=4:expandtab
 --
@@ -53,16 +53,15 @@ module Network.Protocol.HTTP.DAV (
 import Network.Protocol.HTTP.DAV.TH
 
 import Control.Applicative (liftA2, Alternative, Applicative)
-import Control.Error (EitherT(..))
+import Control.Monad.Trans.Except (ExceptT(..), throwE, runExceptT)
 import Control.Lens ((^.), (.=), (%=), (.~))
 import Control.Monad (when, MonadPlus)
 import Control.Monad.Base (MonadBase(..))
-import Control.Monad.Catch (bracket, bracketOnError, catchJust, finally, throwM, mask, uninterruptibleMask, MonadCatch, MonadMask, MonadThrow)
+import Control.Monad.Catch (bracket, bracketOnError, catchJust, finally, throwM, mask, uninterruptibleMask, MonadCatch, MonadThrow)
 import qualified Control.Monad.Catch as MonadCatch
-import Control.Monad.Error (MonadError)
+import Control.Monad.Except (MonadError, catchError, throwError)
 import Control.Monad.Fix (MonadFix)
 import Control.Monad.Trans.Class (lift, MonadTrans)
-import Control.Monad.Trans.Either (left)
 import Control.Monad.IO.Class (liftIO, MonadIO)
 import Control.Monad.State (evalStateT, runStateT, get, MonadState, StateT)
 
@@ -75,7 +74,7 @@ import qualified Data.Map as Map
 
 import Data.Maybe (catMaybes, fromMaybe)
 
-import Network.HTTP.Client (RequestBody(..), httpLbs, parseUrl, applyBasicAuth, Request(..), Response(..), newManager, closeManager, HttpException(..), BodyReader, withResponse, path)
+import Network.HTTP.Client (RequestBody(..), httpLbs, parseUrl, applyBasicAuth, Request(..), Response(..), newManager, HttpException(..), BodyReader, withResponse, path)
 import Network.HTTP.Client.TLS (tlsManagerSettings)
 import Network.HTTP.Types (hContentType, Method, Status, RequestHeaders, unauthorized401, conflict409)
 
@@ -88,25 +87,12 @@ import Data.CaseInsensitive (mk)
 instance Default DAVContext where
     def = DAVContext [] def B.empty B.empty [] Nothing def Nothing "hDav-using application"
 
-newtype DAVT m a = DAVT { runDAVT :: EitherT String (StateT DAVContext m) a }
+newtype DAVT m a = DAVT { runDAVT :: ExceptT String (StateT DAVContext m) a }
     deriving (Alternative, Applicative, Functor, Monad, MonadBase b, MonadError String, MonadFix, MonadIO, MonadPlus, MonadState DAVContext)
 
--- this orphan instance is probably a bad idea
-instance MonadMask m => MonadMask (EitherT e m) where
-    mask a = EitherT $ mask $ \u -> runEitherT (a $ q u)
-        where q u = EitherT . u . runEitherT
-    uninterruptibleMask a = EitherT $ uninterruptibleMask $ \u -> runEitherT (a $ q u)
-        where q u = EitherT . u . runEitherT
-
 instance MonadCatch m => MonadCatch (DAVT m) where
     catch (DAVT m) f = DAVT $ MonadCatch.catch m (runDAVT . f)
 
-instance MonadMask m => MonadMask (DAVT m) where
-    mask a = DAVT $ mask $ \u -> runDAVT (a $ q u)
-        where q u = DAVT . u . runDAVT
-    uninterruptibleMask a = DAVT $ uninterruptibleMask $ \u -> runDAVT (a $ q u)
-        where q u = DAVT . u . runDAVT
-
 instance MonadThrow m => MonadThrow (DAVT m) where
     throwM = lift . throwM
 
@@ -118,7 +104,7 @@ type DAVURL = String
 evalDAVT :: MonadIO m => DAVURL -> DAVT m a -> m (Either String a)
 evalDAVT u f = do
     ctx <- mkDAVContext u
-    r <- (evalStateT . runEitherT . runDAVT) f ctx
+    r <- (evalStateT . runExceptT . runDAVT) f ctx
     closeDAVContext ctx
     return r
 
@@ -128,14 +114,15 @@ mkDAVContext u = liftIO $ do
     req <- liftIO $ parseUrl u
     return $ def { _baseRequest = req, _httpManager = Just mgr }
 
+{-# DEPRECATED closeDAVContext "deprecated because http-client deprecated closeManager" #-}
 closeDAVContext :: MonadIO m => DAVContext -> m ()
-closeDAVContext ctx = liftIO $ maybe (return ()) closeManager (ctx ^. httpManager)
+closeDAVContext _ = return ()
 
-withDAVContext :: (MonadIO m, MonadMask m) => DAVURL -> (DAVContext -> m a) -> m a
-withDAVContext u = bracket (mkDAVContext u) closeDAVContext
+withDAVContext :: MonadIO m => DAVURL -> (DAVContext -> m a) -> m a
+withDAVContext u f = mkDAVContext u >>= f
 
 runDAVContext :: MonadIO m => DAVContext -> DAVT m a -> m (Either String a, DAVContext)
-runDAVContext ctx f = (runStateT . runEitherT . runDAVT) f ctx
+runDAVContext ctx f = (runStateT . runExceptT . runDAVT) f ctx
 
 setCreds :: MonadIO m => B.ByteString -> B.ByteString -> DAVT m ()
 setCreds u p = basicusername .= u >> basicpassword .= p
@@ -167,7 +154,7 @@ davRequest meth addlhdrs rbody = go =<< mkDavRequest meth addlhdrs rbody
   where
     go req = do
       ctx <- get
-      maybe (DAVT $ left "Can't perform request without manager") (liftIO . httpLbs req) (ctx ^. httpManager)
+      maybe (DAVT $ throwE "Can't perform request without manager") (liftIO . httpLbs req) (ctx ^. httpManager)
 
 matchStatusCodeException :: Status -> HttpException -> Maybe ()
 matchStatusCodeException want (StatusCodeException s _ _)
@@ -202,16 +189,13 @@ unlockResource = do
     d <- get
     case _lockToken d of
         Nothing -> return ()
-	Just tok -> do let ahs = [(mk "Lock-Token", tok)]
+        Just tok -> do let ahs = [(mk "Lock-Token", tok)]
                        _ <- davRequest "UNLOCK" ahs emptyBody
                        lockToken .= Nothing
 
 supportsLocking :: DAVContext -> Bool
 supportsLocking = liftA2 (&&) ("LOCK" `elem`) ("UNLOCK" `elem`) . _allowedMethods
 
-supportsCalDAV :: DAVContext -> Bool
-supportsCalDAV = ("calendar-access" `elem`) . _complianceClasses
-
 getPropsM :: MonadIO m => DAVT m XML.Document
 getPropsM = do
     let ahs = [(hContentType, "application/xml; charset=\"utf-8\"")]
@@ -230,7 +214,7 @@ withContentM :: MonadIO m => (Response BodyReader -> IO a) -> DAVT m a
 withContentM handleresponse = do
     req <- mkDavRequest "GET" [] emptyBody
     ctx <- get
-    maybe (DAVT $ left "Can't handle response without manager") (\mgr -> liftIO $ withResponse req mgr handleresponse) (ctx ^. httpManager)
+    maybe (DAVT $ throwE "Can't handle response without manager") (\mgr -> liftIO $ withResponse req mgr handleresponse) (ctx ^. httpManager)
 
 -- | Note that the entire request body is buffered in memory; not suitable
 -- for large files.
@@ -288,8 +272,8 @@ props2patch = XML.renderLBS XML.def . patch . props . fromDocument
        root [] = propertyupdate []
        root prop = propertyupdate
            [ XML.NodeElement $ XML.Element "D:set" Map.empty
-	     [ XML.NodeElement $ XML.Element "D:prop" Map.empty prop ]
-	   ]
+             [ XML.NodeElement $ XML.Element "D:prop" Map.empty prop ]
+           ]
        propertyupdate = XML.Element "D:propertyupdate" (Map.fromList [("xmlns:D", "DAV:")])
        blacklist = [ "{DAV:}creationdate"
                    , "{DAV:}displayname"
@@ -311,21 +295,22 @@ caldavReportM = do
 getOptionsOnce :: MonadIO m => DAVT m ()
 getOptionsOnce = getOptions -- this should only happen once
 
-withLockIfPossible :: (MonadIO m, MonadBase IO m, MonadCatch m, MonadMask m) => Bool -> DAVT m a -> DAVT m a
+withLockIfPossible :: (MonadIO m, MonadBase IO m) => Bool -> DAVT m a -> DAVT m a
 withLockIfPossible nocreate f = do
     getOptionsOnce
     o <- get
     when (supportsLocking o) (lockResource nocreate)
-    f `finally` when (supportsLocking o) unlockResource
+    res <- f
+    when (supportsLocking o) unlockResource
+    return res
 
-withLockIfPossibleForDelete :: (MonadIO m, MonadBase IO m, MonadCatch m, MonadMask m) => Bool -> DAVT m a -> DAVT m a
+withLockIfPossibleForDelete :: (MonadIO m, MonadBase IO m) => Bool -> DAVT m a -> DAVT m a
 withLockIfPossibleForDelete nocreate f = do
     getOptionsOnce
     o <- get
-    let lock = when (supportsLocking o) (lockResource nocreate)
+    when (supportsLocking o) (lockResource nocreate)
     -- a successful delete destroys locks, so only unlock on error
-    let unlock = when (supportsLocking o) unlockResource
-    bracketOnError lock (const unlock) (const f)
+    catchError f (\e -> when (supportsLocking o) unlockResource >> throwError e)
 
 propname :: XML.Document
 propname = XML.Document (XML.Prologue [] Nothing []) root []
@@ -382,7 +367,7 @@ inDAVLocation f a = do
     let r = ctx ^. baseRequest
         r' = r { path = adjustpath r }
         ctx' = baseRequest .~ r' $ ctx
-    lift $ either error return =<< (evalStateT . runEitherT . runDAVT) a ctx'
+    lift $ either error return =<< (evalStateT . runExceptT . runDAVT) a ctx'
   where
     adjustpath = UTF8B.fromString . f . UTF8B.toString . path
 
